# Generate the OTBConfig.cmake file in the build tree.  Also configure
# one for installation.  The file tells external projects how to use
# OTB.

#-----------------------------------------------------------------------------
# Settings specific to the build tree.

# The "use" file.
SET(OTB_USE_FILE ${OTB_BINARY_DIR}/UseOTB.cmake)

# The library dependencies file.
SET(OTB_LIBRARY_DEPENDS_FILE ${OTB_BINARY_DIR}/OTBLibraryDepends.cmake)

# The build settings file.
SET(OTB_BUILD_SETTINGS_FILE ${OTB_BINARY_DIR}/OTBBuildSettings.cmake)

# Library directory.
SET(OTB_LIBRARY_DIRS_CONFIG ${OTB_LIBRARY_PATH})



#            SET(OTB_LIBRARY_DIRS_CONFIG ${OTB_LIBRARY_DIRS_CONFIG} ${GDAL_LIBRARY_DIRS} ${ITK_LIBRARY_DIRS} ${FLTK_LIBRARY_DIRS})





# Determine the include directories needed.
SET(OTB_INCLUDE_DIRS_CONFIG
  ${OTB_INCLUDE_DIRS_BUILD_TREE}
  ${OTB_INCLUDE_DIRS_BUILD_TREE_CXX}
  ${OTB_INCLUDE_DIRS_SYSTEM} 
)

# Built Configure FLTK commands and directories
IF(OTB_USE_EXTERNAL_FLTK)
        SET(OTB_FLTK_FLUID_EXECUTABLE ${FLTK_FLUID_EXECUTABLE})
ELSE(OTB_USE_EXTERNAL_FLTK)
        SET(OTB_FLTK_FLUID_EXECUTABLE ${FLUID_COMMAND})
ENDIF(OTB_USE_EXTERNAL_FLTK)

SET(OTB_FLUID_COMMAND  ${FLUID_COMMAND})
SET(OTB_FLTK_LIBRARY_DIRS ${FLTK_LIBRARY_DIRS})
SET(OTB_FLTK_EXECUTABLE_DIRS ${FLTK_EXECUTABLE_DIRS})

#-----------------------------------------------------------------------------
# Configure OTBConfig.cmake for the BUILD tree.
CONFIGURE_FILE(${OTB_SOURCE_DIR}/OTBConfig.cmake.in
               ${OTB_BINARY_DIR}/OTBConfig.cmake @ONLY IMMEDIATE)

#-----------------------------------------------------------------------------
# Settings specific to the install tree.

# The library dependencies file.
SET(OTB_LIBRARY_DEPENDS_FILE "\${OTB_INSTALL_PREFIX}${OTB_INSTALL_PACKAGE_DIR}/OTBLibraryDepends.cmake")

# The "use" file.
SET(OTB_USE_FILE \${OTB_INSTALL_PREFIX}${OTB_INSTALL_PACKAGE_DIR}/UseOTB.cmake)

# The build settings file.
SET(OTB_BUILD_SETTINGS_FILE \${OTB_INSTALL_PREFIX}${OTB_INSTALL_PACKAGE_DIR}/OTBBuildSettings.cmake)


# Include directories.
SET(OTB_INCLUDE_DIRS_CONFIG \${OTB_INSTALL_PREFIX}${OTB_INSTALL_INCLUDE_DIR})
FOREACH(DIR ${OTB_INCLUDE_RELATIVE_DIRS})
  LIST(APPEND OTB_INCLUDE_DIRS_CONFIG \${OTB_INSTALL_PREFIX}${OTB_INSTALL_INCLUDE_DIR}/${DIR})
ENDFOREACH(DIR)
IF(OTB_INCLUDE_DIRS_SYSTEM)
  LIST(APPEND OTB_INCLUDE_DIRS_CONFIG ${OTB_INCLUDE_DIRS_SYSTEM})
ENDIF(OTB_INCLUDE_DIRS_SYSTEM)

# Add absolue includes paths
SET(OTB_INCLUDE_DIRS_CONFIG ${OTB_INCLUDE_DIRS_CONFIG}
        ${OTB_INCLUDE_ABSOLUE_DIRS}  )

# Include directories.
#       SET(OTB_INCLUDE_DIRS_CONFIG 
#         ${OTB_INCLUDE_DIRS_INSTALL_TREE}
#         ${OTB_INCLUDE_DIRS_SYSTEM}
#       )

# Link directories.
SET(OTB_LIBRARY_DIRS_CONFIG "\${OTB_INSTALL_PREFIX}${OTB_INSTALL_LIB_DIR}")


#            SET(OTB_LIBRARY_DIRS_CONFIG ${OTB_LIBRARY_DIRS_CONFIG} ${GDAL_LIBRARY_DIRS} ${ITK_LIBRARY_DIRS} ${FLTK_LIBRARY_DIRS})



# Install Configure FLTK commands and directories
IF(OTB_USE_EXTERNAL_FLTK)
        SET(OTB_FLTK_FLUID_EXECUTABLE ${FLTK_FLUID_EXECUTABLE})
        SET(OTB_FLUID_COMMAND  ${FLUID_COMMAND})
        SET(OTB_FLTK_LIBRARY_DIRS ${FLTK_LIBRARY_DIRS})
        SET(OTB_FLTK_EXECUTABLE_DIRS ${FLTK_EXECUTABLE_DIRS})
ELSE(OTB_USE_EXTERNAL_FLTK)
        SET(OTB_BINARY_DIRS_CONFIG "\${OTB_INSTALL_PREFIX}${OTB_INSTALL_BIN_DIR}")
        GET_FILENAME_COMPONENT(FLUID_FILE_NAME "${FLUID_COMMAND}" NAME )

        SET(OTB_FLUID_COMMAND ${OTB_BINARY_DIRS_CONFIG}/${FLUID_FILE_NAME})
        SET(OTB_FLTK_FLUID_EXECUTABLE ${OTB_FLUID_COMMAND})
        SET(OTB_FLTK_LIBRARY_DIRS ${OTB_LIBRARY_DIRS_CONFIG})
        SET(OTB_FLTK_EXECUTABLE_DIRS ${OTB_BINARY_DIRS_CONFIG})

#        GET_FILENAME_COMPONENT(FLUID_FILE_NAME "${FLUID_COMMAND}" NAME )
#        SET(OTB_FLUID_COMMAND ${CMAKE_INSTALL_PREFIX}/bin/${FLUID_FILE_NAME})
#        SET(OTB_FLTK_FLUID_EXECUTABLE ${OTB_FLUID_COMMAND})
#        SET(OTB_FLTK_LIBRARY_DIRS ${CMAKE_INSTALL_PREFIX}/lib)
#        SET(OTB_FLTK_EXECUTABLE_DIRS ${CMAKE_INSTALL_PREFIX}/bin)
ENDIF(OTB_USE_EXTERNAL_FLTK)


#-----------------------------------------------------------------------------
# Configure OTBConfig.cmake for the INSTALL tree.

# Construct the proper number of GET_FILENAME_COMPONENT(... PATH)
# calls to compute the installation prefix.
STRING(REGEX REPLACE "/" ";" OTB_INSTALL_PACKAGE_DIR_COUNT
  "${OTB_INSTALL_PACKAGE_DIR}")
SET(OTB_CONFIG_CODE "
# Compute the installation prefix from this ITKConfig.cmake file location.
GET_FILENAME_COMPONENT(OTB_INSTALL_PREFIX \"\${CMAKE_CURRENT_LIST_FILE}\" PATH)")
FOREACH(p ${OTB_INSTALL_PACKAGE_DIR_COUNT})
  SET(OTB_CONFIG_CODE
    "${OTB_CONFIG_CODE}\nGET_FILENAME_COMPONENT(OTB_INSTALL_PREFIX \"\${OTB_INSTALL_PREFIX}\" PATH)"
    )
ENDFOREACH(p)

CONFIGURE_FILE(${OTB_SOURCE_DIR}/OTBConfig.cmake.in
               ${OTB_BINARY_DIR}/Utilities/OTBConfig.cmake @ONLY IMMEDIATE)
